#-*- coding: utf-8 -*-

from gurobipy import *

#Generation of the system
#The variables x are the ones we are interested here
#So a solution will be [x[0],..,x[7]]
#The variables c are only here to modelize the OR constrains
m = Model()
x = [m.addVar(name="x"+str(i),vtype=GRB.BINARY) for i in range(8)]
c = [m.addVar(name="c"+str(i),vtype=GRB.BINARY) for i in range(4)]

m.addGenConstrOr(c[0], [x[0],x[4]])
m.addGenConstrOr(c[1], [x[1],x[5]])
m.addGenConstrOr(c[2], [x[2],x[7]])
m.addGenConstrOr(c[3], [x[3],x[6]])

m.addConstr(c[0] + c[1] + c[2] + c[3] >= 2)
print "System generated"

m.Params.OutputFlag = 0
m.Params.PoolSolutions = 256
m.Params.PoolSearchMode = 2
print "Multiple sol. parameters are set"
m.write("model.mps")
m.write("model.lp")
print "Model written in model.mps and model.lp"

m.optimize()




#To Gurobi Google Groups :
#The following part is used to check each solution and compare against the solutions that Gurobi found
#It is used to generate the files theoricSolutions.txt, foundSolutions.txt and missingSolutions.txt
#The files should be included in the archive anyway

#Generate all 8bit vectors
import itertools
F256 = list(itertools.product([0, 1], repeat=8))

#Retrieve the solutions found by Gurobi (217 instead of 243)
foundSol = []
for i in range(m.SolCount):
	m.Params.SolutionNumber = i
	foundSol.append([int(x[j].Xn) for j in range(8)])
    
#Retrieve the vectors that Gurobi saw as not solution (39 instead of 13)
FoundNotSol = [_x for _x in F256 if _x not in foundSol]


#Now for each 8bit vector, check explicitely if it is a solution
#Basically, this is done by adding constrains like x[i] == rhs (rhs is 0 or 1) and check feasibility

#Generate the constrain and keep them to modify the RHS accordingly to each possible solution vector
cst = [m.addConstr(x[i] == 0) for i in range(8)]
m.update()

realSol = [] #Real solutions of the system
realNotSol = [] #Real remaining vector (not solution)

for v in F256: #for each 8bit vector v
	for i in range(8): #fix the values of x[i] to v[i]
			cst[i].RHS = v[i]
	m.optimize()
	if m.STATUS == 3:
		#then v is not a solution to the system
		realNotSol.append(v)
	else :
		realSol.append(v)


    
#Write the files

#Theoric solution
print "Number of theoric solutions : " + str(len(realSol))
f = open("theoricSolutions.txt", "w")
for v in realSol:
	f.write(str(v)+"\n")
f.close()

#Solutions found by Gurobi
print "Number of solutions found by Gurobi : " + str(len(foundSol))
f = open("foundSolutions.txt", "w")
for v in foundSol:
	f.write(str(v)+"\n")
f.close()

#Missing solutions
print "Number of missing solutions : " + str(len(realSol) - len(foundSol))
f = open("missingSolutions.txt", "w")
for v in realSol:
	if v not in foundSol:
		f.write(str(v)+"\n")

